# # 导入 Flask 的模块
from flask import Blueprint, request

# from data.data import cases
from orm_study.orm_connect import db_session, TestCase

case_bp = Blueprint("用例管理", __name__, url_prefix="/testcase")


# 查询用例
@case_bp.route("/", methods=["GET"])
def get_case():
    args = request.args
    _id = args.get("id")
    datas = []
    if _id:
        case = db_session.query(TestCase).filter_by(id=_id).first()
        # for case in my_cases:
        #     if int(_id) == case.get("id"):
        #         datas.append(case)
        print(f"查询到的数据为=====>{case}")
        if case:
            datas.append({
                "id": case.id,
                "name": case.name,
                "desc": case.desc,
            })
    else:
        # datas = my_cases
        cases = db_session.query(TestCase).all()
        datas = [{"id": case.id, "name": case.name, "desc": case.desc} for case in cases]
        # # 具体的实现
        # for case in cases:
        #     dict_a = {}
        #     dict_a["id"] = case.id
        #     dict_a["name"] = case.name
        #     dict_a["desc"] = case.desc
        #     datas.append(dict_a)
    return {"code": 0, "msg": "success", "data": datas}


#
# 新增测试用例
@case_bp.route("/add", methods=["POST"])
def add_case():
    case_data = request.json
    print(f"接收到的参数<===={case_data}")
    # _id = int(case_data.get("id"))
    name = case_data.get("name")
    desc = case_data.get("desc")
    # new_case = {"id": _id, "name": name, "desc": desc}
    instance = TestCase(name=name, desc=desc)
    # 判断重复
    # for case in my_cases:
    #     if case.get("id") == _id or case.get("name") == name:
    #         return {"code": 40002, "msg": "case is exists"}
    # my_cases.append(new_case)
    # print(f"当前的全部用例: {my_cases}")
    # return {"code": 0, "msg": "add success"}
    if db_session.query(TestCase).filter_by(name=name).first():
        return {"code": 40002, "msg": "case is exists"}
    db_session.add(instance)
    db_session.commit()
    _id = instance.id
    db_session.close()
    if _id:
        return {"code": 0, "msg": "add success", "data": {"case_id": _id}}
    else:
        return {"code": 40001, "msg": "add failed"}


#
#
# 删除测试用例
@case_bp.route("/delete", methods=["DELETE"])
def delete_case():
    _id = int(request.args.get("id"))
    exist = db_session.query(TestCase).filter_by(id=_id).first()
    print(f"exist====> {exist}")
    if exist:
        db_session.query(TestCase).filter_by(id=_id).delete()
        db_session.commit()
        db_session.close()
        return {"code": 0, "msg": "delete success"}
    else:
        return {"code": 40003, "msg": "case is not exists"}


# 更新测试用例
@case_bp.route("/update", methods=["PUT"])
def update_case():
    case_data = request.json
    _id = case_data.get("id")
    name = case_data.get("name")
    desc = case_data.get("desc")
    # 如果case不存在，返回修改失败
    if db_session.query(TestCase).filter_by(id=_id).first():
        # 查到了case之后   进行更新操作
        new_case_data = {
            "name": name,
            "desc": desc,
        }
        db_session.query(TestCase).filter_by(id=_id).update(new_case_data)
        db_session.commit()
        db_session.close()
        return {"code": 0, "msg": "update success"}
    return {"code": 40003, "msg": "case is not exists"}

