from datetime import datetime

from passlib.handlers.sha2_crypt import sha256_crypt
from sqlalchemy import *
from sqlalchemy.orm import declarative_base, sessionmaker, Session

# 先创建基类，使用declarative_base()方法声明一个基类
# 创建表的时候需要继承这个基类对象
Base = declarative_base()
# 创建引擎，通过这个引擎，可以创建数据库连接，比如：mysql，sqlite...
# engine = create_engine("sqlite:///test.db")
## windows 时，有可能会报错，需要加上这个参数
# engine = create_engine('sqlite:///test.db', connect_args={'check_same_thread': False}, echo=True)
db_host = "127.0.0.1"
db_port = "3306"
db_name = "test_platform"
db_user = "root"
db_psw = "hogwarts"
# 拼接url
db_url = f"mysql+pymysql://{db_user}:{db_psw}@{db_host}:{db_port}/{db_name}"
engine = create_engine(db_url, echo=True)

# 连接的格式  dialect+driver://username:password@host:port/database
# sqlite: sqlite:///test.db
# mysql+pymysql://username:password@host:port/database

# 定义db_session对象，可以实现对数据库的增删改查，以及表的创建和删除
DBSession = sessionmaker(bind=engine)
db_session: Session = DBSession()


# 类--对应--表
class User(Base):
    __tablename__ = "user"
    # 属性 -- 对应 -- 表的列属性
    id = Column(Integer, primary_key=True)
    username = Column(String(80), nullable=False, unique=False)
    password = Column(String(120), nullable=False)
    create_time = Column(DateTime, nullable=True, default=datetime.now().strftime("%Y-%m-%d %H:%M:%S"))

    def __init__(self, *args, **kwargs):
        self.username = kwargs.get("username")
        self.password = sha256_crypt.hash(kwargs.get("password"))

    def check_hash_password(self, raw_password):
        return sha256_crypt.verify(raw_password, self.password)

    def __repr__(self):
        return '<User %r>' % self.username

    def as_dict(self):
        return {
            "id": self.id,
            "username": self.username,
            "password": self.password,
            "create_time": str(self.create_time),
        }


class TestCase(Base):
    __tablename__ = "testcase"
    id = Column(Integer, primary_key=True)
    name = Column(String(100), nullable=False)
    desc = Column(String(100))

    def __repr__(self):
        return '<Case %r>' % self.name

    def as_dict(self):
        return {
            "id": self.id,
            "name": self.name,
            "desc": self.desc,
        }


if __name__ == '__main__':
    # 创建表，创建存储在元素数据中所有的表
    Base.metadata.create_all(engine)
    # 删除表
    # Base.metadata.drop_all(engine)

    # 数据的增加操作   create  C
    # 增 - 新增单条数据
    # instance = TestCase(name="用例2", desc="description1")
    # db_session.add(instance)
    # id = instance.id

    # #  批量添加操作
    # add_case1 = TestCase(name="用例3", desc="描述3")
    # add_case2 = TestCase(name="用例4", desc="描述4")
    # ## 方式1  依次添加到session
    # db_session.add(add_case1)
    # db_session.add(add_case2)
    # ## 方式2 一次性添加多条case
    # add_case3 = TestCase(name="用例5", desc="描述5")
    # add_case4 = TestCase(name="用例6", desc="描述6")
    # db_session.add_all([add_case3, add_case4])
    # db_session.commit()
    # db_session.close()
    # 查询操作   R
    # 格式  db_session.query("类").all()/.first()
    # res = db_session.query(TestCase).all()
    # for rs in res:
    #     print(rs)
    # res = db_session.query(TestCase).first()
    # print(res)
    ## 条件查询  db_session.query("类").filter_by(条件).all()/.first()
    # data  = db_session.query(TestCase).filter_by(id=3).first()
    # data  = db_session.query(TestCase).filter_by(id=3).all()
    # print(data)
    # data = db_session.query(TestCase).filter_by(id=3).filter_by(name="用例3").first()
    # print(data)
    # 数据的修改  U
    # 第一种方式，更新某个字段
    # case = db_session.query(TestCase).filter_by(id=1).first()
    # case.desc = "描述信息1111111"
    # # 提交
    # db_session.commit()
    # # 关闭session
    # db_session.close()
    #
    # # 第二种方式，直接调用update方法更新行数据
    # db_session.query(TestCase).filter_by(id=1).update({"name": "用例1111111"})
    # # 提交
    # db_session.commit()
    # # 关闭session
    # db_session.close()
    # 删除  D
    # 方式一：
    # # 查询数据
    # case = db_session.query(TestCase).filter_by(id=1).first()
    # # 删除操作
    # db_session.delete(case)
    # # 提交
    # db_session.commit()
    # # 关闭session
    # db_session.close()
    #
    # # 方式二：
    # # 查询结果直接删除操作
    # db_session.query(TestCase).filter_by(id=2).delete()
    # # 提交
    # db_session.commit()
    # # 关闭session
    # db_session.close()
