import time

import pytest
import requests


class TestCaseServer:

    def setup_class(self):
        self.base_url = "http://127.0.0.1:5000/testcase"

    def add_case(self):
        data = {
            "name": f"用例{time.time()}",
            "desc": f"描述{time.time()}"
        }
        res = requests.post(url=f"{self.base_url}/add", json=data)
        return res.json().get("data", {}).get("case_id")

    def test_add(self):
        data = {
            "name": f"用例{time.time()}",
            "desc": f"描述{time.time()}"
        }
        res = requests.post(url=f"{self.base_url}/add", json=data)
        print(res.json())
        assert res.json().get("code") == 0

    def test_search_all(self):
        res = requests.get(url=self.base_url)
        print(res.json())
        assert res.json().get("code") == 0

    @pytest.mark.parametrize("_id,exp_length", [(3, 1), (1, 0)])
    def test_search_one(self, _id, exp_length):
        res = requests.get(url=self.base_url, params={"id": _id})
        print(res.json())
        assert res.json().get("code") == 0
        length = len(res.json().get("data"))
        assert length == exp_length

    def test_delete(self):
        case_id = self.add_case()
        # case_id = 1
        print(case_id)
        assert case_id
        url = f"{self.base_url}/delete"
        res = requests.delete(url, params={"id": case_id})
        print(res.json())
        assert res.json().get("code") == 0

    def test_update(self):
        case_id = self.add_case()
        print(case_id)
        assert case_id
        url = f"{self.base_url}/update"
        case = requests.get(url=self.base_url, params={"id": case_id}).json().get("data")[0]
        print(case)
        case.update(
            {
                "name": case.get("name") + "_update",
                "desc": case.get("desc") + "_update",
            }
        )
        print(case)
        res = requests.put(url=url, json=case)
        print(res.json())
        assert res.json().get("code") == 0
